﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Linq;
using System.Collections.Generic;

namespace WoodFoxWeiQi.UI
{
	public class WeiQiModel
	{
		// 必要的初始化动作
		public void Init()
		{
			// 只使用 1~19 的下标
			stones = new Stone[20, 20];
			for (var i = 1; i <= 19; i++)
			{
				for (var j = 1; j <= 19; j++)
				{
					stones[i, j] = Stone.None;
				}
			}

			steps = new List<Position>();
			
			// 默认黑棋先走
			NextStone = Stone.Black;

			GameStatus = GameStatus.Ready;
		}

		Stone[,] stones;

		// 游戏状态
		public GameStatus GameStatus
		{
			get;
			set;
		}

		// 保存棋步
		List<Position> steps;

		// 得到棋步的列表
		public List<Position> GetSteps()
		{
			return steps;
		}

		// 得到某一步棋的位置
		public Position GetStep(int step)
		{
			// 越界
			if (step < 1 || step > steps.Count)
				throw new Exception("step index out of range: " + step);

			return steps[step - 1];
		}

		// 得到当前已经进行的棋步数
		public int GetStepCount()
		{
			return steps.Count;
		}

		// 查找某个位置对应的最后的棋步数
		public int GetLastStepNumberByPosition(Position pos)
		{
			if (GetStepCount() == 0) return 0;

			for (int i = steps.Count; i >= 1; i--)
			{
				var p = steps[i - 1];
				if (p.X == pos.X && p.Y == pos.Y)
				{
					return i;
				}
			}
			return 0;
		}

		// 下一个该谁下棋
		public Stone NextStone
		{
			get;
			set;
		}

		// 完成当前落子步骤。
		// 这里会执行轮换玩家颜色等逻辑
		private void FinishTurn()
		{
			if (NextStone == Stone.Black)
				NextStone = Stone.White;
			else if (NextStone == Stone.White)
				NextStone = Stone.Black;
		}

		// 取得某个位置的棋子
		public Stone GetStone(int x, int y)
		{
			return stones[x, y];
		}

		public Stone GetStone(Position pos)
		{
			return GetStone(pos.X, pos.Y);
		}

		// 在某点尝试落子
		public void SetStone(Stone stone, int x, int y)
		{
			if (!CanMove(stone, x,y))
				return;

			stones[x,y] = stone;

			// 保存棋步
			var pos = new Position(x, y);
			steps.Add(pos);

			// 切换玩家
			FinishTurn();

			var positions = new List<Position> { pos };

			// 触发更新
			OnBoardUpdated(positions);
		}

		public void SetStone(Stone stone, Position pos)
		{
			SetStone(stone, pos.X, pos.Y);
		}

		// 如果未指定棋子颜色，则使用下一个棋子的颜色
		public void SetStone(Position pos)
		{
			SetStone(NextStone, pos);
		}

		public void SetStone(int x, int y)
		{
			SetStone(NextStone, x, y);
		}

		public event EventHandler<BoardUpdateEventArgs> BoardUpdated;

		// 通知事件发生
		public void OnBoardUpdated(List<Position> positions)
		{
			if (BoardUpdated != null)
			{
				BoardUpdated(this, new BoardUpdateEventArgs {
					ChangedPositions = positions
				});
			}
		}
		
		// 判断某个点是否能落子
		//
		// 在如下情况下，可能会导致落子失败：
		//		1. 坐标出界
		//		2. 该位置已有棋子
		//		3. 打劫，没有找劫材就提劫。
		//		4. 属于自杀，并且不能提取对方的棋子。
		private bool CanMove(Stone stone, int x, int y)
		{
			// 1. 坐标出界
			if (!CheckPosition(x, y))
				return false;

			// 2. 该位置已有棋子
			if (GetStone(x, y) != Stone.None)
				return false;

			// 3. 打劫判断

			// 4. 自杀判断

			// TODO...


			return true;
		}

		private bool CanMove(Stone stone, Position pos)
		{
			return CanMove(stone, pos.X, pos.Y);
		}

		// 检查某个坐标是否出界
		private bool CheckPosition(int x, int y)
		{
			if (x < 1 || x > 19)
				return false;
			if (y < 1 || y > 19)
				return false;

			return true;
		}

		// 检查某个坐标是否出界
		private bool CheckPosition(Position pos)
		{
			return CheckPosition(pos.X, pos.Y);
		}
	}
}
